#version 120

#define FOG_ENABLED_NETHER //Enables fog in the nether

uniform float frameTimeCounter;
uniform mat4 gbufferModelView;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D depthtex0;
uniform sampler2D gcolor;
uniform sampler2D noisetex;
uniform vec3 fogColor;

varying vec2 texcoord;

const int noiseTextureResolution = 64;
const float invNoiseRes = 1.0 / float(noiseTextureResolution);

float square(float x)        { return x * x; } //faster than pow().

float fogify(float x, float width) { return width / (x * x + width); } //fast, vaguely bell curve-shaped function with variable width

#ifdef FOG_ENABLED_NETHER
	vec3 calcFogColor(vec3 pos) {
		float n = square(texture2D(noisetex, frameTimeCounter * vec2(0.21562, 0.19361) * invNoiseRes).r) - 0.1;
		if (n > 0.0) {
			vec3 brightFog = vec3(fogColor.r * (n + 1.0), mix(fogColor.g, fogColor.r, n), fogColor.b);
			return mix(fogColor, brightFog, fogify(dot(pos, gbufferModelView[1].xyz), 0.125));
		}
		else return fogColor;
	}
#endif

void main() {
	vec3 color = texture2D(gcolor, texcoord).rgb;

	//no skybasic program in the nether. use deferred instead.
	#ifdef FOG_ENABLED_NETHER
		vec3 pos = vec3(texcoord, texture2D(depthtex0, texcoord).r);
		if (pos.z == 1.0) {
			vec4 v = gbufferProjectionInverse * vec4(pos * 2.0 - 1.0, 1.0);
			//pos = v.xyz / v.w;
			color = calcFogColor(normalize(v.xyz));
		}
	#endif

/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, 1.0); //gcolor
}