#version 120

//#define CUBIC_CHUNKS //Disables black fog/sky colors below Y=0
#define TF_AURORAS //Adds auroras to the sky in the twilight forest
#define TF_SKY_FIX //Enable this if the sky looks wrong in the twilight forest

uniform float frameTimeCounter;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D depthtex0;
uniform sampler2D gcolor;
uniform sampler2D noisetex;
uniform vec3 cameraPosition;
uniform vec3 fogColor;
uniform vec3 skyColor;

varying vec2 texcoord;

const int noiseTextureResolution = 64;
const float invNoiseRes = 1.0 / float(noiseTextureResolution);

float square(float x)        { return x * x; } //faster than pow().

float interpolateSmooth1(float x) { return x * x * (3.0 - 2.0 * x); }

float fogify(float x, float width) { return width / (x * x + width); } //fast, vaguely bell curve-shaped function with variable width

vec3 hue(float h) {
	h = fract(h) * 6.0;
	return clamp(
		vec3(
			abs(h - 3.0) - 1.0,
			2.0 - abs(h - 2.0),
			2.0 - abs(h - 4.0)
		),
		0.0,
		1.0
	);
}

vec3 calcFogColor(vec3 pos) {
	#ifndef CUBIC_CHUNKS
		if (cameraPosition.y < -gbufferModelViewInverse[3][1]) return vec3(0.0);
	#endif

	return mix(skyColor, fogColor, fogify(max(dot(pos, gbufferModelView[1].xyz), 0.0), 0.0625));
}

void main() {
	vec3 color = texture2D(gcolor, texcoord).rgb;

	float depth = texture2D(depthtex0, texcoord).r;

	if (depth == 1.0) {
		vec3 pos = vec3(texcoord, depth);
		vec4 v = gbufferProjectionInverse * vec4(pos * 2.0 - 1.0, 1.0);
		pos = v.xyz / v.w;
		float dist = length(pos);

		#ifdef TF_SKY_FIX
			color = calcFogColor(pos / dist);
		#endif

		#ifdef TF_AURORAS
			#ifndef CUBIC_CHUNKS
				if (cameraPosition.y > -gbufferModelViewInverse[3][1]) {
			#endif
					vec3 worldPos = normalize((gbufferModelViewInverse * vec4(pos, 0.0)).xyz);
					float auroraBrightness = 1.0 - abs(worldPos.y - 0.5) * 3.0;
					if (worldPos.y > 0.0 && auroraBrightness > 0.0) {
						vec2 auroraStart = worldPos.xz / worldPos.y * invNoiseRes;
						vec2 auroraStep = auroraStart * -0.5;
						float dither = fract(dot(gl_FragCoord.xy, vec2(0.25, 0.5))) * 0.0625;
						float time = frameTimeCounter * invNoiseRes;
						vec3 auroraColor = vec3(0.0);
						for (int i = 0; i < 16; i++) {
							vec2 auroraPos = (i * 0.0625 + dither) * auroraStep + auroraStart;
							float noise = 1.0 - abs(texture2D(noisetex, vec2(auroraPos.x * 0.5 + (time * 0.03125), auroraPos.y * 2.0)).r * 10.0 - 5.0); //primary noise layer, defines the overall shape of auroras
							if (noise > 0.0) {
								noise *= square(texture2D(noisetex, vec2(auroraPos.x * 16.0, auroraPos.y * 16.0 + (time * 0.5))).r * 2.0 - 1.0); //secondary noise layer, adds detail to the auroras
								auroraColor += hue(texture2D(noisetex, auroraPos * 3.0 + (time * 0.1875)).r * 0.5 + 0.35) * noise * square(1.0 - abs(float(i) * 0.125 - 1.0)); //tertiary noise layer, defines the color of the auroras
							}
						}
						color += sqrt(auroraColor) * 0.75 * interpolateSmooth1(auroraBrightness);
					}
			#ifndef CUBIC_CHUNKS
				}
			#endif
		#endif
	}

/* DRAWBUFFERS:06 */
	gl_FragData[0] = vec4(color, 1.0); //gcolor
	gl_FragData[1] = vec4(depth, 0.0, 0.0, 1.0); //gaux3
}