#version 120

#include "/lib/defines.glsl"

uniform float blindness;
uniform float frameTimeCounter;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D depthtex0;
uniform sampler2D gcolor;
uniform sampler2D noisetex;
uniform vec3 cameraPosition;
        vec3 eyePosition = cameraPosition + gbufferModelViewInverse[3].xyz; //because cameraPosition isn't actually the position of the camera -_-

varying vec2 texcoord;

#include "/lib/noiseres.glsl"

#include "/lib/goldenOffsets.glsl"

#include "/lib/math.glsl"

#include "/lib/hue.glsl"

#include "/lib/endEffects.glsl"

#ifdef VOID_CLOUDS
	#include "lib/drawVoidClouds.glsl"
#endif

void main() {
	vec3 color = texture2D(gcolor, texcoord).rgb;

	vec3 pos = vec3(texcoord, texture2D(depthtex0, texcoord).r);
	bool sky = pos.z == 1.0;
	vec4 v = gbufferProjectionInverse * vec4(pos * 2.0 - 1.0, 1.0);
	pos = v.xyz / v.w;
	float dist = length(pos);
	vec3 posNorm = pos / dist;
	vec3 worldPos = (gbufferModelViewInverse * vec4(pos, 1.0)).xyz + cameraPosition;

	if (sky) {
		#if defined(ENDER_NEBULAE) || defined(ENDER_STARS)
			vec3 worldPosNorm = normalize(worldPos - eyePosition);
			vec2 skyPos = worldPosNorm.xz / (worldPosNorm.y + 1.0);
			//adding 1.0 to posNorm.y is how I get the sky to "wrap" around you.
			//if you want to visualize the effect this has, I would suggest setting color to vec3(fract(skyPos), 0.0).
			float multiplier = 8.0 / (lengthSquared2(skyPos) + 8.0); //wrapping behavior produces a mathematical singularity below you, so we just reduce the opacity there to hide that.

			#ifdef ENDER_NEBULAE
				vec4 cloudclr = drawNebulae(skyPos);
				color = mix(color, cloudclr.rgb, cloudclr.a * multiplier);
			#endif

			#ifdef ENDER_STARS
				vec3 starclr = drawStars(skyPos);
				color += starclr * multiplier;
			#endif
		#endif

		color *= 1.0 - blindness;
	}

	#ifdef VOID_CLOUDS
		float cloudDiff = VOID_CLOUD_HEIGHT - eyePosition.y;
		vec3 baseCloudPos = worldPos - eyePosition;
		bool cloudy = sign(cloudDiff) == sign(baseCloudPos.y);

		if (cloudy) {
			baseCloudPos = normalize(baseCloudPos);
			baseCloudPos = vec3(baseCloudPos.xz * (cloudDiff / baseCloudPos.y), cloudDiff).xzy;
			float opacityModifier = -1.0;

			if (!sky && dist * dist < lengthSquared3(baseCloudPos)) { //terrain in front of the clouds
				opacityModifier = abs(worldPos.y - VOID_CLOUD_HEIGHT) / 4.0;
				if (opacityModifier < 1.0) { //in the fadeout range
					baseCloudPos = worldPos - eyePosition;
				}
				else {
					cloudy = false;
				}
			}

			if (cloudy) {
				vec4 cloudclr = drawVoidClouds(baseCloudPos, opacityModifier);

				if (cloudclr.a > 0.001) {
					if (opacityModifier > 0.0 && opacityModifier < 1.0) { //in the fadeout range
						cloudclr.a *= interpolateSmooth1(opacityModifier);
					}
					color = mix(color, cloudclr.rgb, cloudclr.a);
				}
			}
		}
	#endif

/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, 1.0); //gcolor
}