#version 120

#include "/lib/defines.glsl"

uniform float far;
uniform float frameTimeCounter;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform sampler2D noisetex;

varying float heldLightDistModifier; //Zoom in with optifine to increase range of held lights.
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 blockLightColor; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.

#ifdef CROSS_PROCESS
	varying vec3 crossProcessColor; //Color to use for cross-processing
#endif

varying vec3 pos;
varying vec4 glcolor;
varying vec4 heldLightColor; //Color of held light source. Alpha = brightness.

#include "/lib/noiseres.glsl"

#include "/lib/calcHeldLightColor.glsl"

void main() {
	pos         = (gl_ModelViewMatrix  * gl_Vertex).xyz;
	gl_Position =  gl_ProjectionMatrix * vec4(pos, 1.0);
	texcoord    = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord     = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	glcolor     =  gl_Color;

	#include "lib/colors.glsl"

	vec3 realNormal = (gbufferModelViewInverse * vec4(normalize(gl_NormalMatrix * gl_Normal), 0.0)).xyz;
	float glmult = dot(vec4(abs(realNormal.x), abs(realNormal.z), max(realNormal.y, 0.0), max(-realNormal.y, 0.0)), vec4(0.6, 0.8, 1.0, 0.5));
	glmult = mix(glmult, 1.0, lmcoord.x * lmcoord.x); //increase brightness when block light is high
	glcolor.rgb *= glmult;
}